@@ -44,13 +44,27 @@ output$ui_dtree_remove <- renderUI({
   actionButton("dtree_remove", "Remove", icon = icon("trash"), class = "btn-danger")
 })
 
+dtreeIsNum <- function(x) {
+  if (!grepl("[A-Za-z]+", x)) {
+    x <- try(eval(parse(text = x), envir = r_data), silent = TRUE)
+    if (inherits(x, "try-error")) {
+      FALSE
+    } else {
+      if (sshhr(is.na(as.numeric(x)))) FALSE else TRUE
+    }
+  } else {
+    FALSE
+  }
+}
+
 output$ui_dtree_sense_name <- renderUI({
   dte <- dtree_run()
-  if (length(dte) < 2) return(HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables."))
+  mess <- HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables.")
+  if (!inherits(dte, "list")) return(mess)
   vars <- dte$yl$variables
-  if (is_empty(vars)) return(HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables."))
-  vars <- vars[!is.na(sshhr(sapply(vars, as.numeric)))]
-  if (length(vars) == 0) return(HTML("No variables are available for sensitivity analysis. If the input file does contain a variables section, press the Calculate button to show the list of available variables."))
+  if (is_empty(vars)) return(mess)
+  vars <- vars[!is.na(sshhr(sapply(vars, dtreeIsNum)))]
+  if (length(vars) == 0) return(mess)
   vars[names(vars)] <- names(vars)
 
   selectInput(
@@ -62,21 +76,24 @@ output$ui_dtree_sense_name <- renderUI({
 
 output$ui_dtree_sense_decision <- renderUI({
   dte <- dtree_run()
-  if (length(dte) < 2) return(invisible())
-  jl <- dte$jl
-  if (is.null(jl)) return(invisible())
-
-  ## all decisions in the tree
-  decs <-
-    jl$Get(function(x) if (length(x$parent$decision) > 0) x$payoff) %>%
-    na.omit() %>%
-    names()
+  if (inherits(dte, "list") && !is.null(dte[["jl"]])) {
+    ## all decisions in the tree
+    decs <-
+      dte$jl$Get(function(x) if (length(x$parent$decision) > 0) x$payoff) %>%
+      na.omit() %>%
+      names()
+  } else {
+    decs <- ""
+  }
 
   selectizeInput(
     "dtree_sense_decision", label = "Decisions to evaluate:",
     choices = decs, multiple = TRUE,
     selected = state_multiple("dtree_sense_decision", decs, decs),
-    options = list(plugins = list("remove_button"))
+    options = list(
+      placeholder = "Select decisions to evaluate",
+      plugins = list("remove_button")
+    )
   )
 })
 
@@ -142,7 +159,7 @@ output$dtree <- renderUI({
         mode = "yaml",
         theme = getOption("radiant.ace_theme", default = "tomorrow"),
         wordWrap = TRUE,
-        debounce = 100,
+        debounce = 0,
         height = "auto",
         value = state_init("dtree_edit", dtree_example) %>% gsub("\t", "    ", .),
         placeholder = "Provide structured input for a decision tree. Then click the\n\"Calculate tree\" button to generate results. Click the ? icon\non the top left of your screen for help and examples",
@@ -276,19 +293,14 @@ observe({
 
 dtree_name <- function() {
   isolate({
-    dtree_name <- gsub("[^ A-z0-9_\\.\\-]", " ", input$dtree_name) %>% gsub("\\s{2,}", " ", .) %>% gsub("(^\\s+)|(\\s+$)", "", .)
+    dtree_name <- input$dtree_name
     if (is_empty(dtree_name)) {
-      dtree_name <- stringr::str_match(input$dtree_edit, "\\s*name:\\s*(.*)\\n\\s*type:")[2]
+      dtree_name <- stringr::str_match(input$dtree_edit, "^\\s*name:\\s*(.*)\\n")[2]
       if (is.na(dtree_name)) {
         dtree_name <- "dtree"
-      } else {
-        dtree_name %<>% tolower %>%
-          gsub("[^[:alnum:] ]", "", .) %>%
-          gsub("\\s+", "_", .) %>%
-          gsub("^([0-9]+)", ".", .)
       }
     }
-    dtree_name
+    fix_names(dtree_name)
   })
 }
 
@@ -301,9 +313,20 @@ dtree_run <- eventReactive(vals_dtree$dtree_edit_hotkey > 1, {
   ## update settings and get data.tree name
   dtree_name <- dtree_namer()
 
+  ## ensure correct spacing
+  yl <- gsub(":([^ $])", ": \\1", input$dtree_edit) %>%
+    gsub(":[ ]{2,}", ": ", .) %>%
+    gsub(":[ ]+\\n", ":\n", .) %>%
+    gsub("[ ]*\\n[ ]+\\n[ ]*", "\n\n", .) %>%
+    gsub(":\\s*([-]{0,1})(\\.[0-9]+\\s*\\n)", ": \\10\\2", ., perl = TRUE)
+
+  yl <- gsub("(\\n[ ]+)([0-9]+)", "\\1_\\2", yl)
+
+  shinyAce::updateAceEditor(session, "dtree_edit", value = yl)
+
   if (input$dtree_edit != "") {
     withProgress(message = "Creating decision tree", value = 1, {
-      dtree(input$dtree_edit, opt = input$dtree_opt)
+      dtree(yl, opt = input$dtree_opt, envir = r_data)
     })
   }
 })
@@ -328,7 +351,7 @@ dtree_plot_inputs <- reactive({
 })
 
 output$dtree_plot <- DiagrammeR::renderDiagrammeR({
-  if (is_empty(input$dtree_final)) return(invisible())
+  req(input$dtree_final)
   dt <- dtree_run()
   if (is.null(dt)) {
     invisible()
@@ -340,18 +363,16 @@ output$dtree_plot <- DiagrammeR::renderDiagrammeR({
 
 ## Evaluate tree sensitivity
 .plot_dtree_sensitivity <- eventReactive(input$dtree_run_sensitivity, {
-  if (is_not(input$dtree_sense_decision)) {
-    return("At least one decision should be selected for evaluation")
+  if (is_empty(input$dtree_sense_decision)) {
+    "At least one decision should be selected for evaluation"
+  } else if (is_empty(input$dtree_sense)) {
+    "No variables were specified for evaluation.\nClick the + icon to add variables for sensitivity evaluation"
+  } else {
+    withProgress(
+      message = "Conducting sensitivity analysis", value = 1,
+      sensitivity(dtree_run(), gsub("\n+", "", input$dtree_sense), input$dtree_sense_decision, envir = r_data, shiny = TRUE)
+    )
   }
-
-  if (is_empty(input$dtree_sense)) {
-    return("No variables were specified for evaluation.\nClick the + icon to add variables for sensitivity evaluation")
-  }
-
-  withProgress(
-    message = "Conducting sensitivity analysis", value = 1,
-    sensitivity(dtree_run(), gsub("\n+", "", input$dtree_sense), input$dtree_sense_decision, shiny = TRUE)
-  )
 })
 
 dtree_sense_width <- reactive({
@@ -359,11 +380,19 @@ dtree_sense_width <- reactive({
 })
 
 dtree_sense_height <- eventReactive(input$dtree_run_sensitivity, {
-  if (is_empty(input$dtree_sense, "")) 650 else length(strsplit(input$dtree_sense, ";")[[1]]) * 400
+  if (is_empty(input$dtree_sense, "")) {
+    650
+  } else {
+    strsplit(input$dtree_sense, ";") %>%
+      unlist() %>%
+      unique() %>%
+      length() * 400
+  }
 })
 
 output$plot_dtree_sensitivity <- renderPlot({
   req(input$dtree_run_sensitivity, cancelOutput = TRUE)
+  req(input$dtree_sense_name, cancelOutput = TRUE)
   isolate({
     .plot_dtree_sensitivity() %>% {
       if (is.character(.)) {
@@ -401,21 +430,19 @@ observeEvent(input$dtree_load_yaml, {
   ## remove characters that may cause problems in shinyAce
   yaml_file %<>% gsub("[\x80-\xFF]", "", .) %>% gsub("\r", "\n", .)
 
-  dtree_name <- sub(paste0(".", tools::file_ext(inFile$name)), "", inFile$name)
+  dtree_name <- sub(paste0(".", tools::file_ext(inFile$name)), "", inFile$name) %>%
+    fix_names()
   r_data[[dtree_name]] <- yaml_file
   if (!bindingIsActive(as.symbol(dtree_name), env = r_data)) {
     shiny::makeReactiveBinding(dtree_name, env = r_data)
   }
-  # r_data[["dtree_list"]] <- c(dtree_name, r_data[["dtree_list"]]) %>% unique()
   r_info[["dtree_list"]] <- c(dtree_name, r_info[["dtree_list"]]) %>% unique()
   updateSelectInput(session = session, inputId = "dtree_list", selected = dtree_name)
   shinyAce::updateAceEditor(session, "dtree_edit", value = gsub("\t", "    ", yaml_file))
 })
 
 observeEvent(input$dtree_list, {
-  dtree_name <- gsub("[^ A-z0-9_\\.\\-]", " ", input$dtree_name) %>%
-    gsub("\\s{2,}", " ", .) %>%
-    gsub("(^\\s+)|(\\s+$)", "", .)
+  dtree_name <- fix_names(input$dtree_name)
   if (is_empty(dtree_name)) dtree_name <- dtree_name()
   r_data[[dtree_name]] <- input$dtree_edit
 
@@ -444,17 +471,25 @@ observeEvent(input$dtree_edit, {
 })
 
 dtree_namer <- reactive({
-  dtree_name <- fix_names(input$dtree_name)
+  dtree_name_org <- input$dtree_name
 
-  if (is_empty(dtree_name)) dtree_name <- fix_names(input$dtree_list[1])
-  if (is_empty(dtree_name)) dtree_name <- fix_names(dtree_name())
+  if (is_empty(dtree_name_org)) {
+    dtree_name <- input$dtree_list[1]
+    if (is_empty(dtree_name)) {
+      dtree_name <- dtree_name()
+    } else {
+      dtree_name <- fix_names(dtree_name)
+    }
+  } else {
+    dtree_name <- fix_names(dtree_name_org)
+  }
 
   r_data[[dtree_name]] <- input$dtree_edit
-  r_info[["dtree_list"]] <- c(dtree_name, r_info[["dtree_list"]]) %>% unique()
+  r_info[["dtree_list"]] <- c(dtree_name, setdiff(r_info[["dtree_list"]], dtree_name_org)) %>% unique()
   if (!bindingIsActive(as.symbol(dtree_name), env = r_data)) {
     shiny::makeReactiveBinding(dtree_name, env = r_data)
   }
-  updateSelectInput(session = session, inputId = "dtree_list", selected = dtree_name)
+  updateSelectInput(session = session, inputId = "dtree_list", selected = dtree_name, choices = r_info[["dtree_list"]])
   dtree_name
 })
 
@@ -483,7 +518,6 @@ observeEvent(input$dtree_remove, {
 
   ## update settings and get data.tree name
   dtree_name <- dtree_namer()
-  id <- sample(seq_len(1000000), 1)
   xcmd <- clean_args(dtree_plot_inputs(), dtree_plot_args[-1]) %>%
     deparse(control = getOption("dctrl"), width.cutoff = 500L) %>%
     {if (. == "list()") {
@@ -497,7 +531,7 @@ observeEvent(input$dtree_remove, {
   inp <- list(yl = dtree_name)
   if (input$dtree_opt == "min") inp$opt <- "min"
 
-  update_report(
+  ret <- update_report(
     inp_main = inp,
     fun_name = "dtree",
     inp_out = inp_out,
@@ -507,10 +541,13 @@ observeEvent(input$dtree_remove, {
     fig.height = dtree_sense_height(),
     xcmd = xcmd
   )
+
+  ret
 })
 
+
 dl_dtree_save <- function(path) {
-  capture.output(dtree(input$dtree_edit) %>%
+  capture.output(dtree(input$dtree_edit, envir = r_data) %>%
     summary(input = FALSE, output = TRUE)) %>%
     cat(file = path, sep = "\n")
 }
